@@ -19,8 +19,12 @@ test_that("Test adding point", {
 
   P1 <- matrix(c(1,0.5), nrow=1,ncol=2)
   p <- sp::SpatialPoints(matrix(c(0,0.5),1,2))
+
+  dat = sp::SpatialPointsDataFrame(p,  data.frame(y=1))
   
-  graph$add_observations(Spoints = p, data = data.frame(y=1))
+  dat <- sf::st_as_sf(dat)
+
+  graph$add_observations(data = dat)
   graph$observation_to_vertex()
 
   expect_equal(graph$V, matrix(c(0, 0, 1, 0, 0, 1, 1, 0.5), 4, 2), tol=1e-9)
