@@ -26,12 +26,12 @@ test_that("Test resistance metric", {
     df_test <- data.frame(y = y, edge_number = PtE[,1], distance_on_edge = PtE[,2])
 
   graph$add_observations(data = df_test, normalized = TRUE)
-  graph$compute_resdist()
+  graph$compute_resdist(full = TRUE)
   D1 <- graph$res_dist[[1]]
   expect_equal(as.vector(D1), as.vector(distances[reo,reo]), tolerance = 1e-10)
 
   theta <- c(1, 2, 3)
-  lik1 <- likelihood_graph_covariance(graph, model = "isoCov", y_graph = graph$get_data()$y, repl=NULL, cov_function = exp_covariance, log_scale=FALSE, maximize = TRUE)
+  lik1 <- MetricGraph:::likelihood_graph_covariance(graph, model = "isoCov", y_graph = graph$get_data()$y, repl=NULL, cov_function = exp_covariance, log_scale=FALSE, maximize = TRUE)
   lik1 <- lik1(theta)
 
   Sigma <- theta[2]^2 * exp(-theta[3]*distances)
@@ -42,7 +42,7 @@ test_that("Test resistance metric", {
 
 
   graph$observation_to_vertex()
-  lik2 <- likelihood_graph_covariance(graph, model = "isoCov", cov_function = exp_covariance, y_graph = graph$get_data()$y, repl=NULL, log_scale = FALSE, maximize = TRUE)
+  lik2 <- MetricGraph:::likelihood_graph_covariance(graph, model = "isoCov", cov_function = exp_covariance, y_graph = graph$get_data()$y, repl=NULL, log_scale = FALSE, maximize = TRUE)
   lik2 <- lik2(theta)
 
   PtE.order <- PtE[reo, ]
@@ -50,12 +50,12 @@ test_that("Test resistance metric", {
   graph <- metric_graph$new(lines = Lines)
   df_temp <- data.frame(y = y.order, edge_number = PtE.order[,1], distance_on_edge = PtE.order[,2])
   graph$add_observations(data=df_temp, normalized = TRUE)
-  graph$compute_resdist()
-  lik3 <- likelihood_graph_covariance(graph,  model = "isoCov", repl = NULL, y = graph$get_data()$y, cov_function = exp_covariance, log_scale = FALSE, maximize = TRUE)
+  graph$compute_resdist(full = TRUE)
+  lik3 <- MetricGraph:::likelihood_graph_covariance(graph,  model = "isoCov", repl = NULL, y = graph$get_data()$y, cov_function = exp_covariance, log_scale = FALSE, maximize = TRUE)
   lik3 <- lik3(theta)
 
   graph$observation_to_vertex()
-  lik4 <- likelihood_graph_covariance(graph, model = "isoCov", repl = NULL, y_graph = graph$get_data()$y, cov_function = exp_covariance, log_scale = FALSE, maximize = TRUE)
+  lik4 <- MetricGraph:::likelihood_graph_covariance(graph, model = "isoCov", repl = NULL, y_graph = graph$get_data()$y, cov_function = exp_covariance, log_scale = FALSE, maximize = TRUE)
   lik4 <- lik4(theta)
 
   expect_equal(lik.true, lik1, tolerance = 1e-10)
